import typing

from panther_config import detection

__all__ = ["deep_equal"]


def deep_equal(path: str, value: typing.Any) -> detection.PythonFilter:
    return detection.PythonFilter(
        func=_deep_equal,
        params={"path": path, "expected": value},
    )


def _deep_equal(obj: typing.Any, params: typing.Any) -> bool:
    import functools
    import collections

    path = params["path"]
    keys = path.split(".")
    expected = params["expected"]

    actual = functools.reduce(
        lambda d, key: d.get(key, None) if isinstance(d, collections.Mapping) else None,
        keys,
        obj,
    )

    return bool(actual == expected)



import typing

from panther_config import detection

__all__ = ["ips_in_cidr"]


def ips_in_cidr(cidr: str, path: str = "p_any_ip_addresses") -> detection.PythonFilter:
    return detection.PythonFilter(
        func=_ip_in_cidr,
        params={
            "path": path,
            "cidr": cidr,
        },
    )


def _ip_in_cidr(obj: typing.Any, params: typing.Any) -> bool:
    import ipaddress
    import functools
    import collections

    path = params["path"]
    cidr = ipaddress.ip_network(params["cidr"])

    keys = path.split(".")

    obj_at_path = functools.reduce(
        lambda d, key: d.get(key, None) if isinstance(d, collections.Mapping) else None,
        keys,
        obj,
    )

    if obj_at_path is None:
        raise RuntimeError(f"no value found at path '{path}'")

    if isinstance(obj_at_path, str):
        return ipaddress.ip_address(obj_at_path) in cidr

    if isinstance(obj_at_path, collections.Iterable):
        for ip in obj_at_path:
            if ipaddress.ip_address(ip) in cidr:
                return True

        return False

    raise RuntimeError(f"IP value at path '{path}' was not a string or iterable")



from pathlib import Path
from setuptools import setup, find_packages

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
version = (this_directory / "VERSION").read_text()

setup(
    name='panther_utils',
    url="https://panther.com",
    author="Panther Labs Inc.",
    author_email="support@panther.io",
    version=version,
    packages=find_packages(),
    python_requires=">=3.9",
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords='security detection',
    install_requires=[
        'panther_core>=0.0.6,<0.1.0',
    ],
    classifiers=[
        'Development Status :: 1 - Planning',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Topic :: Security',
        'Typing :: Typed',
        'Programming Language :: Python :: 3',
    ]
)



from panther_config import testing
from panther_utils import network_filters


class TestNetworkFilters(testing.PantherPythonFilterTestCase):
    def test_ips_in_cidr(self) -> None:
        test_filter = network_filters.ips_in_cidr("10.0.0.0/8")

        self.assertFilterIsValid(test_filter)

        self.assertFilterMatches(test_filter, {"p_any_ip_addresses": ["10.0.0.1"]})
        self.assertFilterNotMatches(
            test_filter, {"p_any_ip_addresses": ["192.168.0.1"]}
        )

    def test_ips_in_cidr_custom_path(self) -> None:
        test_filter = network_filters.ips_in_cidr("10.0.0.0/8", path="nested.ips")

        self.assertFilterIsValid(test_filter)

        self.assertFilterMatches(test_filter, {"nested": {"ips": ["10.0.0.1"]}})
        self.assertFilterNotMatches(test_filter, {"nested": {"ips": ["192.168.0.1"]}})

    def test_ips_in_cidr_non_array(self) -> None:
        test_filter = network_filters.ips_in_cidr("10.0.0.0/8", path="nested.ips")

        self.assertFilterIsValid(test_filter)

        self.assertFilterMatches(test_filter, {"nested": {"ips": "10.0.0.1"}})
        self.assertFilterNotMatches(test_filter, {"nested": {"ips": "192.168.0.1"}})

    def test_ips_in_cidr_no_value_at_path(self) -> None:
        test_filter = network_filters.ips_in_cidr("10.0.0.0/8", path="bad.path.to.ips")

        self.assertFilterIsValid(test_filter)

        self.assertRaises(
            RuntimeError,
            lambda: self.assertFilterMatches(
                test_filter, {"nested": {"ips": "10.0.0.1"}}
            ),
        )

    def test_ips_in_cidr_bad_value_at_path(self) -> None:
        test_filter = network_filters.ips_in_cidr("10.0.0.0/8", path="nested.field")

        self.assertFilterIsValid(test_filter)

        self.assertRaises(
            RuntimeError,
            lambda: self.assertFilterMatches(test_filter, {"nested": {"field": True}}),
        )



from panther_config import testing
from panther_utils import match_filters


class TestMatchFilters(testing.PantherPythonFilterTestCase):
    def test_deep_equal(self) -> None:

        test_filter = match_filters.deep_equal("a.b", "targeted-value")

        self.assertFilterIsValid(test_filter)

        self.assertFilterMatches(test_filter, {"a": {"b": "targeted-value"}})
        self.assertFilterNotMatches(test_filter, {"a": {"b": "other-value"}})
